import java.awt.*;
import java.awt.image.BufferedImage;

public class MapWithSetWarshipsGeneric <T extends IDrawingObject,U extends AbstractMap> {
    private final int _pictureWidth;
    private final int _pictureHeight;
    private final int _placeSizeWidth = 120;
    private final int _placeSizeHeight = 50;
    private final SetWarshipsGeneric<T> _setWarships;
    private final U _map;

    public MapWithSetWarshipsGeneric(int picWidth, int picHeight, U map)
    {
        int width = picWidth / _placeSizeWidth;
        int height = picHeight/_placeSizeHeight;
        _setWarships = new SetWarshipsGeneric<T>(width * height);
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;
        _map = map;
    }

    public int add(T warship)
    {
        return _setWarships.Insert(warship);
    }

    public T remove(int position)
    {
        return _setWarships.Remove(position);
    }

    public Image ShowSet()
    {
        BufferedImage bmp = new BufferedImage(_pictureWidth, _pictureWidth,BufferedImage.TYPE_INT_RGB);
        Graphics gr = bmp.getGraphics();
        DrawBackground(gr);
        DrawWarship(gr);
        return bmp;
    }

    public Image ShowOnMap()
    {
        Shaking();
        for (int i = 0; i < _setWarships.getCount(); i++)
        {
            var warship = _setWarships.Get(i);
            if (warship != null)
            {
                return _map.CreateMap(_pictureWidth, _pictureHeight, warship);
            }
        }
        return new BufferedImage(_pictureWidth, _pictureHeight,BufferedImage.TYPE_INT_RGB);
    }

    public Image MoveObject(Direction direction)
    {
        if (_map != null)
        {
            return _map.MoveObject(direction);
        }
        return new BufferedImage(_pictureWidth, _pictureHeight,BufferedImage.TYPE_INT_RGB);
    }

    public void Shaking()
    {
        int j = _setWarships.getCount() - 1;
        for (int i = 0; i < _setWarships.getCount(); i++)
        {
            if (_setWarships.Get(i) == null)
            {
                for (; j > i; j--)
                {
                    var warship = _setWarships.Get(j);
                    if (warship != null)
                    {
                        _setWarships.Insert(warship, i);
                        _setWarships.Remove(j);
                        break;
                    }
                }
                if (j <= i)
                {
                    return;
                }
            }
        }
    }

    private void DrawBackground(Graphics gr)
    {
        Graphics2D g=(Graphics2D)gr;

        Color brush=Color.BLACK;
        Stroke penWide = new BasicStroke(5);
        Stroke penThin = new BasicStroke(1);

        for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
        {
            for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; ++j)
            {
                g.setColor(brush);
                g.setStroke(penWide);
                g.drawLine(i * _placeSizeWidth + 20, j * _placeSizeHeight+2, i * _placeSizeWidth + (int)(_placeSizeWidth*0.8), j * _placeSizeHeight+2);
                g.drawLine(i * _placeSizeWidth + 20, j * _placeSizeHeight + _placeSizeHeight/2+2, i * _placeSizeWidth + (int)(_placeSizeWidth * 0.8), j * _placeSizeHeight + _placeSizeHeight/2+2);
                g.drawLine(i * _placeSizeWidth + (int)(_placeSizeWidth * 0.8), j * _placeSizeHeight + 2, i * _placeSizeWidth + _placeSizeWidth, j * _placeSizeHeight + _placeSizeHeight/2);
                g.drawLine(i * _placeSizeWidth + _placeSizeWidth, j * _placeSizeHeight + _placeSizeHeight / 2, i * _placeSizeWidth+ (int)(_placeSizeWidth * 0.8), j * _placeSizeHeight + _placeSizeHeight);
                g.setStroke(penThin);
            }
        }
    }

    private void DrawWarship(Graphics gr)
    {
        int width = _pictureWidth / _placeSizeWidth;
        int height = _pictureHeight / _placeSizeHeight;

        for (int i = 0; i < _setWarships.getCount(); i++)
        {
            if (_setWarships.Get(i) != null)
            {
                _setWarships.Get(i).SetObject(i % width * _placeSizeWidth, (height - 1 - i / width) * _placeSizeHeight, _pictureWidth, _pictureHeight);
                _setWarships.Get(i).DrawingObject(gr);
            }
        }
    }
}
